package org.openxma.dsl.dom.naming;

import java.util.Collections;

import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.naming.DefaultDeclarativeQualifiedNameProvider;
import org.eclipse.xtext.util.IResourceScopeCache;
import org.eclipse.xtext.util.PolymorphicDispatcher;
import org.eclipse.xtext.util.Tuples;
import org.openxma.dsl.dom.DomPackage;

import com.google.common.base.Function;
import com.google.inject.Inject;
import com.google.inject.Provider;

public class DomNameProvider extends DefaultDeclarativeQualifiedNameProvider {
	private PolymorphicDispatcher<String> qualifiedName = new PolymorphicDispatcher<String>("qualifiedName", 1, 1,
			Collections.singletonList(this), PolymorphicDispatcher.NullErrorHandler.<String> get()) {
		@Override
		protected String handleNoSuchMethod(Object... params) {
			return null;
		}
	};

	@Inject
	DomNameFunction domNameFunction;

	@Inject
	private IResourceScopeCache cache = IResourceScopeCache.NullImpl.INSTANCE;

	protected Function<EObject, String> getResolver(EObject eObject) {
		if (null != DomPackage.eINSTANCE.getEClassifier(eObject.eClass().getName())) {
			return getDomNameFunction();
		}
		return super.getResolver();
	}

	private Function<EObject, String> getDomNameFunction() {
		if (domNameFunction == null) {
			domNameFunction = new DomNameFunction();
		}
		return domNameFunction;
	}

	public String getQualifiedName(final EObject obj) {
		return cache.get(Tuples.pair(obj, "fqn"), obj.eResource(), new Provider<String>() {

			public String get() {
				EObject temp = obj;
				String name = qualifiedName.invoke(temp);
				if (name != null)
					return name;
				String value = getResolver(obj).apply(temp);
				if (value == null)
					return null;
				while (temp.eContainer() != null) {
					temp = temp.eContainer();
					String parentsName = getQualifiedName(temp);
					if (parentsName != null)
						return parentsName + getDelimiter() + value;
				}
				return value;
			}

		});

	}

}
